#include <stdio.h>
#include <stdlib.h>

#include "d3.h"

//#define SHOW_ALL_SORTS_OF_INFO

int some_function_user_writes_to_pick_the_answer(int answers)
{
	int v;
	printf("\n> ");
	scanf("%d", &v);
	printf("\n\n");
	return v;
}

int main(int parc, char ** pars)
{
	// Some variables
	int answers;            // Number of answers
	int i;                  // Loop iterator
	int done;               // Loop terminator flag
	const char *q, *a;      // Pointers to question and answer text
	struct d3 *d;           // D3 engine
	struct d3deck *deck;    // A card deck
	char * serialize_buf;
	int len;
	FILE * f;

	if (parc < 2)
	{
		printf("Usage:\n"
			"d3test <dialog filename>\n"
			"The test also reads and writes the datafile tagstore.dat where "
			"the current tag information is stored for the testing of persistent "
			"communications. To reset, just delete the file.\n\n");
		return -1;
	}

	// Init
	d = d3_init();

	// Load a conversation
	deck = d3_load_deck_bin(pars[1]);
	
	if (deck == NULL)
	{
		printf("\"%s\" load failed.\n", pars[1]);
		return -1;
	}

	f = fopen("tagstore.dat", "rb");
	if (f)
	{
		fseek(f,0,SEEK_END);
		len = ftell(f);
		fseek(f,0,SEEK_SET);
		serialize_buf = malloc(len+1);
		fread(serialize_buf, 1, len, f);
		fclose(f);
		serialize_buf[len] = 0;
		d3_deserialize(d, serialize_buf);
		free(serialize_buf);
	}

	// Use it, start from first card (NULL)
	d3_use_deck(d, deck, NULL);

	// We're not done yet
	done = 0;

#ifdef SHOW_ALL_SORTS_OF_INFO
	printf("#  Deck ID: \"%s\"\n", d3_get_deck_id(d));
	printf("#  Deck user data: \"%s\"\n", d3_get_deck_user_data(d));
#endif


	while (!done)
	{
#ifdef SHOW_ALL_SORTS_OF_INFO
		printf("#  Card ID: \"%s\"\n", d3_get_card_id(d));
		printf("#  Card user data: \"%s\"\n", d3_get_card_user_data(d));
		printf("#  Current tags (%d): ", d3_tag_count(d));
		for (i = 0; i < d3_tag_count(d); i++)
			printf("\"%s\" ", d3_tag(d, i));
		printf("\n\n");
#endif
		// Get question text, print it if any
		q = d3_get_question(d);
		if (q != NULL) 
		{ 
			printf("%s\n", q);
		}

		// Find out how many answers we have..
		answers = d3_get_answer_count(d);
		if (answers == 0)
		{
			done = 1; // No answers, we're done here
		}
		else
		{
			printf("\n");
			// If first answer text doesn't exist, it's the auto-choose option  
			if (d3_get_answer(d, 0) == NULL)
			{
				d3_choose(d, 0);
				continue;
			}

			// Otherwise, let's print the answers
			for (i = 0; i < answers; i++)
			{
				a = d3_get_answer(d, i);
				if (a) 
				{
					printf("%d) %s\n", i, a);
				}
				else
				{
					printf("%d) Empty (should not happen, bad data)\n", i);
				}      
			}

			// I could use scanf, but I'd rather not. Let's just say user defines the input.
			i = some_function_user_writes_to_pick_the_answer(answers);

			// and then we pick it.
			d3_choose(d, i);
		}  
	}

	printf("\n\n");

	f = fopen("tagstore.dat", "wb");
	if (f)
	{
		len = d3_serialize_estimate(d);
		serialize_buf = malloc(len);
		d3_serialize(d, serialize_buf, NULL, len);
		fwrite(serialize_buf, 1, len, f);
		fclose(f);
		free(serialize_buf);
	}

	// Free memory
	d3_free_deck(deck);
	d3_deinit(d);

	// All done!
}